]> git.sesse.net Git - cubemap/blobdiff - client.h
Support HTTP/1.1 persistent connections (not that useful yet).
[cubemap] / client.h
index 4e8003110b11c04e20b5a713f44b75cff294133a..ed04bc02f888f85572364d1b363a7ce01b59175a 100644 (file)
--- a/client.h
+++ b/client.h
@@ -7,14 +7,19 @@
 #include <time.h>
 #include <string>
 
+#include "tlse.h"
+
 class ClientProto;
 struct Stream;
 
 // Digested statistics for writing to logs etc.
+// Note that "referer" and "user_agent" here are already escaped for scary characters.
 struct ClientStats {
        std::string url;
        int sock;
        std::string remote_addr;
+       std::string referer;
+       std::string user_agent;
        timespec connect_time;
        size_t bytes_sent;
        size_t bytes_lost;
@@ -38,9 +43,11 @@ struct Client {
 
        // Some information only used for logging.
        std::string remote_addr;
+       std::string referer;
+       std::string user_agent;
 
-       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA, SENDING_ERROR, WAITING_FOR_KEYFRAME, PREBUFFERING };
-       State state;
+       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA, SENDING_SHORT_RESPONSE, WAITING_FOR_KEYFRAME, PREBUFFERING };
+       State state = READING_REQUEST;
 
        // The HTTP request, as sent by the client. If we are in READING_REQUEST,
        // this might not be finished.
@@ -49,16 +56,28 @@ struct Client {
        // What stream we're connecting to; parsed from <request>.
        // Not relevant for READING_REQUEST.
        std::string url;
-       Stream *stream;
+       Stream *stream = nullptr;
+
+       // Whether we should close the connection after sending the response.
+       // Not relevant for READING_REQUEST. Must be true if http_11 is false.
+       bool close_after_response;
+
+       // Whether the request came in over HTTP/1.1 or higher.
+       bool http_11;
 
-       // The header we want to send. This is nominally a copy of Stream::header,
-       // but since that might change on reconnects etc., we keep a local copy here.
-       // Only relevant for SENDING_HEADER or SENDING_ERROR; blank otherwise.
-       std::string header_or_error;
+       // The header we want to send, or the response with headers if we know
+       // it in its entirety after reading the request (typically an error).
+       // This is nominally a copy of Stream::header, but since that might
+       // change on reconnects etc., we keep a local copy here. Only relevant
+       // for SENDING_HEADER or SENDING_SHORT_RESPONSE; blank otherwise.
+       //
+       // Must start with the string "HTTP/1.0 ", which will be changed to 1.1
+       // if relevant.
+       std::string header_or_short_response;
 
        // Number of bytes we've sent of the header. Only relevant for SENDING_HEADER
-       // or SENDING_ERROR.
-       size_t header_or_error_bytes_sent;
+       // or SENDING_SHORT_RESPONSE.
+       size_t header_or_short_response_bytes_sent = 0;
 
        // Number of bytes we are into the stream (ie., the end of last send).
        // -1 means we want to send from the end of the backlog (the normal case),
@@ -66,14 +85,19 @@ struct Client {
        // -2 means we want to send from the _beginning_ of the backlog.
        // Once we go into WAITING_FOR_KEYFRAME, PREBUFFERING or SENDING_DATA,
        // these negative values will be translated to real numbers.
-       size_t stream_pos;
+       size_t stream_pos = 0;
 
        // Number of bytes we've sent of data. Only relevant for SENDING_DATA.
-       size_t bytes_sent;
+       size_t bytes_sent = 0;
 
        // Number of times we've skipped forward due to the backlog being too big,
        // and how many bytes we've skipped over in all. Only relevant for SENDING_DATA.
-       size_t bytes_lost, num_loss_events;
+       size_t bytes_lost = 0, num_loss_events = 0;
+
+       TLSContext *tls_context = nullptr;
+       const unsigned char *tls_data_to_send = nullptr;
+       unsigned tls_data_left_to_send = 0;
+       bool in_ktls_mode = false;
 };
 
 #endif  // !defined(_CLIENT_H)